// Endless Brick Scrolling
// By:	Hata (483)
//		Chrono (1630)
//		with help from Honor (7989)
//		& modified by Demian (10334)

exec("./greekbinds.cs");

// If either of the preferences are not defined.
if($Pref::EBS::AcrossTabs $= "" || $Pref::EBS::AcrossSubcategories $= "")
{
	// Initialize them to their default values.
	$Pref::EBS::AcrossSubcategories = true;
	$Pref::EBS::AcrossTabs = true;
}

if(!$BrickScrolling::Binded)
{
	addKeyBind("Building", "Endless Brick Scrolling", "ScrollBrickModifier");
	$BrickScrolling::Binded = true;
}

function ScrollBrickModifier(%val)
{
	$BrickScrolling::Ready = %val;
	
	// The color the brick selector border turns when the key is held down or white when not in use.
	HUD_BrickActive.setColor($BrickScrolling::Ready ? "1 0.6 0.1 1" : "1 1 1 1");
}

function ScrollBrickActual(%x)
{
	// If brick scrolling is reversed.
	if($pref::input::reversebrickscroll)
		%x *= -1; // Reverse the direction.

	if(%x < 0)
		%move = -1; // Move backwards
	else
		%move = 1; // Move forwards

	// Brick bar slot.
	%slot = getWord(HUD_BrickActive.position,0) / 64;
	
	// Selected brick.
	%data = $invData[%slot];
	
	// The brick subcategory.
	%sub = %data.sub;
	
	// Index in the subcategory. Add/Subtract 1 depending on the direction to move to.
	%index = %sub.id[%data] + %move;
	
	// If new index is less than the index of the first brick.
	if(%index < 0)
	{
		// If scrolling across subcategories
		if ($Pref::EBS::AcrossSubcategories)
		{
			// If this tab has enough subcategories.
			if (%sub.subIndex - 1 >= 0)
			{
				// Get the previous subcategory of this tab.
				%sub = BrickScrollTab.getObject(%sub.tabIndex).getObject(%sub.subIndex - 1);
			}
			else
			{
				// If scrolling across tabs
				if ($Pref::EBS::AcrossTabs)
				{
					// If the brick store has enough tabs.
					if (%sub.tabIndex - 1 >= 0)
					{
						// Get the last subcategory of the previous tab.
						%sub = BrickScrollTab.getObject(%sub.tabIndex - 1).getObject(BrickScrollTab.getObject(%sub.tabIndex - 1).getCount() - 1);
					}
					else
					{
						// Get the last subcategory of the last tab.
						%sub = BrickScrollTab.getObject(BrickScrollTab.getCount() - 1).getObject(BrickScrollTab.getObject(BrickScrollTab.getCount() - 1).getCount() - 1);
					}
				}
				else
				{
					// Get the last subcategory of this tab.
					%sub = BrickScrollTab.getObject(%sub.tabIndex).getObject(BrickScrollTab.getObject(%sub.tabIndex).getCount() - 1);
				}
			}
		}
		
		// Get the last brick of this subcategory.
		%index = %sub.getCount()-1;
	}
	else if(%index >= %sub.getCount())
	{
		// New index is greater than the index of the last brick in this subcategory.
		
		// If scrolling across subcategories
		if ($Pref::EBS::AcrossSubcategories)
		{
			// If this tab has more subcategories.
			if (%sub.subIndex + 1 <= BrickScrollTab.getObject(%sub.tabIndex).getCount() - 1)
			{
				// Get the next subcategory of this tab.
				%sub = BrickScrollTab.getObject(%sub.tabIndex).getObject(%sub.subIndex + 1);
			}
			else
			{
				// If scrolling across tabs
				if ($Pref::EBS::AcrossTabs)
				{
					// If the brick store has more tabs.
					if (%sub.tabIndex + 1 <= BrickScrollTab.getCount() - 1)
					{
						// Get the first subcategory of the next tab.
						%sub = BrickScrollTab.getObject(%sub.tabIndex + 1).getObject(0);
					}
					else
					{
						// Get the first subcategory of the first tab.
						%sub = BrickScrollTab.getObject(0).getObject(0);
					}
				}
				else
				{
					// Get the first subcategory of this tab.
					%sub = BrickScrollTab.getObject(%sub.tabIndex).getObject(0);
				}
			}
		}
		
		// Get the first brick of this subcategory.
		%index = 0;
	}
	
	// New brick.
	%dataNew = %sub.getObject(%index);
	
	// Some magic to keep the brick selected when entering brick store.
	$BSD_InvData[%slot]=%dataNew;
	
	commandToServer('BuyBrick',%slot,%dataNew);
	commandToServer('InstantUsebrick',%dataNew);
	clientCmdServerMessage('MsgSetInvData',"",%slot,%dataNew);
}

package EndlessBrickScrolling
{
	function scrollInventory(%x)
	{
		if($BrickScrolling::Ready)
			ScrollBrickActual(%x);
		else
			Parent::scrollInventory(%x);
	}
	function BSD_LoadBricks()
	{
		if(isObject(BrickScrollTab))
			BrickScrollTab.delete();

		new SimGroup(BrickScrollTab){};
		RootGroup.add(BrickScrollTab);
		
		return Parent::BSD_LoadBricks();
	}
	function BSD_addCategory(%name)
	{
		if(!isObject(BrickScrollTab @ "_" @ %name))
		{
			%sg = new SimGroup(){};
			%sg.tabIndex = BrickScrollTab.getCount();
			BrickScrollTab.add(%sg);
			%sg.setName(BrickScrollTab @ "_" @ %name);
		}
		
		return Parent::BSD_addCategory(%name);
	}
	function BSD_addSubCategory(%cat,%name)
	{
		if(!isObject("BrickScrollSet_" @ %cat @ "_" @ %name))
		{
			%ss = new SimSet(){};
			%co = "BrickScrollTab_" @ %cat;
			%ss.tabIndex = %co.tabIndex;
			%ss.subIndex = %co.getCount();
			%co.add(%ss);
			%ss.setName("BrickScrollSet_" @ %cat @ "_" @ %name);
		}
		
		return Parent::BSD_addSubCategory(%cat,%name);
	}
	function BSD_CreateBrickButton(%data)
	{
		%sub = "BrickScrollSet_" @ %data.category @ "_" @ %data.subcategory;
		
		if(isObject(%sub))
		{
			%sub.id[%data] = %sub.getCount();
			%sub.add(%data);
			%data.sub = %sub;
		}
		
		return Parent::BSD_CreateBrickButton(%data);
	}
};
activatePackage(EndlessBrickScrolling);
